%{

/* Ascii-to-Table version 2.2
**
** A conversion utility to convert gifscii type ASCII-Artwork into
** grayscale HTML 3.0 compliant html documents using tables.
**
** Copyright(C) 1997 by Patrick J.M. Keane --  All rights reserved.
** (pkeane@wilkes.edu)
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

char shade1[4], shade2[4], shade3[4] ;
int reverse=0, widthset=0, width=0 ;
int shade1set=0, shade2set=0, shade3set=0 ;

void maketd(const char *value) {
  printf("<td bgcolor=") ;
  if (!(strcmp(value, "00")))
    printf("#000000>") ;
  else 
    printf("#%s%s%s>",
	   ((shade1set==0) ? value : shade1),
	   ((shade2set==0) ? value : shade2),
	   ((shade3set==0) ? value : shade3)) ;
  printf("<font size=0>&nbsp;</font></td>") ;
}

main(int argc, char *argv[]) {
  int c;
  extern int optind;
  extern char *optarg;
  extern int opterr;

  while ((c = getopt(argc, argv, "vw:r:g:b:xh")) != EOF) {
    switch (c) {
    case 'x':
      reverse = 1 ;
      break;
    case 'v':
      fprintf(stderr, "\na2t - The ASCII to HTML 3.0 table translator version 2.1\n\tby Patrick Keane\n\n") ;
      exit(0) ;
      break ;
    case 'h':
      fprintf(stderr, "Usage:\n\tcat asciifile | a2t [-h] [-x] [-[rgb] value] [-w width] > document.html\n\n") ;
      fprintf(stderr, "\t-h       : This help screen\n") ;
      fprintf(stderr, "\t-x       : Reverse output\n") ;
      fprintf(stderr, "\t-r value : Constant R <R>GB value\n") ;
      fprintf(stderr, "\t-g value : Constant G R<G>B value\n") ;
      fprintf(stderr, "\t-b value : Constant B RG<B> value\n") ;
      fprintf(stderr, "\t-w value : Set width of output table\n") ;
      exit(0) ;
      break;
    case 'r':
      shade1set = 1 ;
      strcpy(shade1, optarg) ;
      break ;
    case 'g':
      shade2set = 1 ;
      strcpy(shade2, optarg) ;
      break ;
    case 'b':
      shade3set = 1 ;
      strcpy(shade3, optarg) ;
      break ;
    case 'w':
      widthset = 1 ;
      width = atoi(optarg) ;
      break ;
    default:
      fprintf(stderr, "Try \"a2t -h\" for more information\n");
      exit(1) ;
      break;
    }
  }

  printf ("<title>Table Art!</title>\n") ; 
  printf ("<!---Generated by a2t version 2.2 by Patrick Keane--->\n") ; 
  printf ("<!---pkeane@wilkes.edu--->\n") ; 
  printf ("<body bgcolor=black text=white>\n") ;
  printf ("<center>\n") ;
  printf ("<table border=0 cellpadding=0 cellspacing=1") ;
  if (widthset)
    printf(" width=%d>", width) ;
  else
    printf(">") ;
  printf ("<tr>\n") ;
  yylex() ;
  printf("</table></center>\n") ;

}

%}

%option yylineno
ws [ ]*
%%

"$"|"@" { (reverse) ? maketd("00") : maketd("ff") ; }
"W"|"M" { (reverse) ? maketd("20") : maketd("f7") ; }
"B"|"%"|"8"|"&" { (reverse) ? maketd("20") : maketd("f0") ; }
"#"|"*"|"9"|"6"|"H" { (reverse) ? maketd("20") : maketd("e7") ; }
"o"|"h"|"k" { (reverse) ? maketd("27") : maketd("e0") ; }
"4"|"5"|"S"|"K" { (reverse) ? maketd("30") : maketd("d7") ; }
"a"|"e"|"s" { (reverse) ? maketd("37") : maketd("d0") ; }
"b"|"d"|"p"|"q" { (reverse) ? maketd("40") : maketd("c7") ; }
"w"|"m"|"3" { (reverse) ? maketd("47") : maketd("b7") ; }
"z"|"O"|"0"|"Q" { (reverse) ? maketd("50") : maketd("b0") ; }
"L"|"G"|"D"|"C"|"2" { (reverse) ? maketd("57") : maketd("a7") ; }
"R"|"E"|"U"|"X" { (reverse) ? maketd("60") : maketd("a0") ; }
"N"|"A"|"Y"|"P" { (reverse) ? maketd("67") : maketd("97") ; }
"F"|"J"|"Z"|"z"|"c" { (reverse) ? maketd("70") : maketd("90") ; }
"g"|"y" { (reverse) ? maketd("77") : maketd("85") ; }
"x"|"v"|"u"|"n" { (reverse) ? maketd("80") : maketd("80") ; }
"="|"I"|"r"|"j"|"T" { (reverse) ? maketd("87") : maketd("77") ; }
"f"|"t" { (reverse) ? maketd("90") : maketd("70") ; }
"|"|"?"|"V"|"/"|"\\"|"7" { (reverse) ? maketd("97") : maketd("67") ; }
"["|"]"|"{"|"}" { (reverse) ? maketd("a0") : maketd("60") ; }
"<"|">"|"("|")" { (reverse) ? maketd("c5") : maketd("50") ; }
"i"|"l"|"1"|"|"|"!" { (reverse) ? maketd("d0") : maketd("40") ; }
":"|";"|"+"|"~" { (reverse) ? maketd("e0") : maketd("30") ; }
"^"|"\"" { (reverse) ? maketd("e7") : maketd("27") ; }
"-"|"_" { (reverse) ? maketd("ff") : maketd("20") ; }
"'"|"`" { (reverse) ? maketd("ff") : maketd("20") ; }
"."|"," { (reverse) ? maketd("ff") : maketd("20") ; }
{ws}"\n" { maketd("00") ;
           printf("</tr>\n") ; }
" " { (reverse) ? maketd("ff") : maketd("00") ; }
.    { fprintf(stderr, "Warning: Character %s is not recognized.\n",
	       yytext) ;
       fprintf(stderr, "Choosing a medium color!\n") ;
       maketd("97") ; }
%%

void yyerror(char *msg) { 
  fprintf(stderr, "^GError :\tLine %d: %s at '%s'\n", yylineno, msg, yytext) ;
}

int yywrap() {
  return (1);
}



